/*
 * Copyright 2017 Arne Vajhj.
 * 
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *    http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License. 
 */

package dk.vajhoej.vms.call.lib;

import static dk.vajhoej.vms.call.lib.VMSCodes.*;

import java.io.UnsupportedEncodingException;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import dk.vajhoej.record.RecordException;

import static dk.vajhoej.vms.call.VMS.*; 

/**
 * Class SYS_Util is a high level wrapper library for system services (SYS$ functions).
 */
public class SYS_Util extends SYS {
	/**
	 * Call SYS$IDTOASC to get list of all identifiers and call callback for each.
	 * @param proc callback
	 * @throws UnsupportedEncodingException if error decoding UIC name
	 */
	public static void SYS$IDTOASC(IDProcessor proc) throws UnsupportedEncodingException {
		ID_CTX ctx = new ID_CTX();
		for(;;) {
			Result<Pair<Integer,String>> temp = ctx.SYS$IDTOASC();
			if(!isSuccess(temp.getStatus())) break;
			proc.process(temp.getResult().getItem1(), temp.getResult().getItem2());
		}
	}
	/**
	 * Call SYS$IDTOASC to get map of identifiers to identifier names.
	 * @param inclneg include negative (identifiers that are not UIC's)
	 * @return map of identifiers to identifier names
	 * @throws UnsupportedEncodingException if error decoding UIC name
	 */
	public static Map<Integer,String> SYS$IDTOASC(boolean inclneg) throws UnsupportedEncodingException {
		final boolean inclneg2 = inclneg;
		final Map<Integer,String> res = new HashMap<Integer,String>();
		SYS$IDTOASC(new IDProcessor() {
			public void process(int id, String idname) {
				if(inclneg2 || id > 0) res.put(id, idname);
			}
		});
		return res;
	}
	private static boolean contains(int[] a, int v) {
		for(int i = 0; i < a.length; i++) {
			if(a[i] == v) {
				return true;
			}
		}
		return false;
	}
	/**
	 * Call SYS$PROCESS_SCAN to get all PID's and call SYS$GETJPIW for each PID to get list of items and call callback for each..
	 * @param code code for SYS$PROCESS_SCAN
	 * @param value value for SYS$PROCESS_SCAN
	 * @param itms array of items
	 * @param proc callback
	 * @throws UnsupportedEncodingException if some string values can not be decoded
	 * @throws RecordException if internal error
	 */
	public static void SYS$PROCESS_SCAN_and_SYS$GETJPIW(int code, String value, int[] itms, PIDProcessor<Map<Integer,Object>> proc) throws UnsupportedEncodingException, RecordException {
		contains(itms, JPI$_PID);
		PID_CTX ctx = new PID_CTX();
		ctx.SYS$PROCESS_SCAN(code, value);
		for(;;) {
			Result<Map<Integer,Object>> temp = ctx.SYS$GETJPIW(itms);
			if(!isSuccess(temp.getStatus())) break;
			proc.process((int)(Integer)temp.getResult().get(JPI$_PID), temp.getResult());
		}
	}
	/**
	 * Call SYS$PROCESS_SCAN to get all PID's and call SYS$GETJPIW for each PID to get list of items.
	 * @param code code for SYS$PROCESS_SCAN
	 * @param value value for SYS$PROCESS_SCAN
	 * @param itms array of items
	 * @return Java List of Java Map with items
	 * @throws UnsupportedEncodingException if some string values can not be decoded
	 * @throws RecordException if internal error
	 */
	public static List<Map<Integer,Object>> SYS$PROCESS_SCAN_and_SYS$GETJPIW(int code, String value, int[] itms) throws UnsupportedEncodingException, RecordException {
		final List<Map<Integer,Object>> res = new ArrayList<Map<Integer,Object>>();
		SYS$PROCESS_SCAN_and_SYS$GETJPIW(code, value, itms, new PIDProcessor<Map<Integer,Object>>() {
			public void process(int pid, Map<Integer,Object> o) {
				res.add(o);
			}
		});
		return res;
	}
}
