/*
 * Copyright 2017 Arne Vajhj.
 * 
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *    http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License. 
 */

package dk.vajhoej.vms.call.script.py;

import java.io.UnsupportedEncodingException;
import java.util.List;
import java.util.Map;

import dk.vajhoej.record.RecordException;
import dk.vajhoej.vms.call.VMS.Block;
import dk.vajhoej.vms.call.VMS.TypeWrapper;
import dk.vajhoej.vms.call.VMS.Word;
import dk.vajhoej.vms.call.script.SYS_Script;

/**
 * Class SYS_Py is a Python friendly wrapper library for system services (SYS$ functions).
 */
public class SYS_Py extends SYS_Script {
	/**
	 * Call SYS$ASCTIM to get string with current time.
	 * @return return status code + string with time
	 * @throws UnsupportedEncodingException if time string can not be decoded
	 */
	public static Result<String> SYS__ASCTIM() throws UnsupportedEncodingException {
		return SYS$ASCTIM();
	}
	/**
	 * Call SYS$ASCTIM to get string with specified time.
	 * @param t VMS time
	 * @return return status code + string with time
	 * @throws UnsupportedEncodingException if time string can not be decoded
	 */
	public static Result<String> SYS__ASCTIM(long t) throws UnsupportedEncodingException {
		return SYS$ASCTIM(t);
	}
	/**
	 * Call SYS$ASSIGN to open channel to device.
	 * @param devnam device name
	 * @return return status code + channel
	 * @throws UnsupportedEncodingException if device name can not be decoded
	 */
	public static Result<Word> SYS__ASSIGN(String devnam) throws UnsupportedEncodingException {
		return SYS$ASSIGN(devnam);
	}
	/**
	 * Call SYS$BINTIM to get specified VMS time.
	 * @param timstr time string
	 * @return return status code + VMS time
	 * @throws UnsupportedEncodingException if time string can not be encoded
	 */
	public static Result<Long> SYS__BINTIM(String timstr) throws UnsupportedEncodingException {
		return SYS$BINTIM(timstr);
	}
	/**
	 * Call LIB$DASSGN to close channel.
	 * @param chan channel
	 * @return return status code
	 */
	public static Result<Void> SYS__DASSGN(Word chan) {
		return SYS$DASSGN(chan);
	}
	/**
	 * Call SYS$GETDVIW to get a list of items.
	 * @param devnam device name
	 * @param itms array of items
	 * @return return status code + Java Map with items
	 * @throws UnsupportedEncodingException if device name can not be encoded or some string values can not be decoded
	 * @throws RecordException if internal error
	 */
	public static Result<Map<String,Object>> SYS__GETDVIW(String devnam, String[] itms) throws UnsupportedEncodingException, RecordException {
		return SYS$GETDVIW(devnam, itms);
	}
	/**
	 * Call SYS$GETJPIW to get a list of items.
	 * @param pid PID
	 * @param itms array of items
	 * @return return status code + Java Map with items
	 * @throws UnsupportedEncodingException if some string values can not be decoded
	 * @throws RecordException if internal error
	 */
	public static Result<Map<String,Object>> SYS__GETJPIW(int pid, String[] itms) throws UnsupportedEncodingException, RecordException {
		return SYS$GETJPIW(pid, itms);
	}
	/**
	 * Call SYS$IDTOASC to get name for identifier.
	 * @param id identifier
	 * @return return status code + identifier name
	 * @throws UnsupportedEncodingException if identifier name can not be decoded
	 */
	public static Result<String> SYS__IDTOASC(int id) throws UnsupportedEncodingException {
		return SYS$IDTOASC(id);
	}
	/**
	 * Class ID_CTX represents the context for SYS$IDTOASC calls.
	 */
	public static class ID_CTX extends dk.vajhoej.vms.call.lib.SYS.ID_CTX {
		/**
		 * Call SYS$IDTOASC to return next identifier.
		 * @return return status code + identifier
		 * @throws UnsupportedEncodingException if some identifier names can not be decoded
		 */
		public Result<Pair<Integer, String>> SYS__IDTOASC() throws UnsupportedEncodingException {
			return SYS$IDTOASC();
		}
	}
	/**
	 * Call SYS$GETUAI to get a list of items.
	 * @param usrnam username
	 * @param itms array of items
	 * @return return status code +  Java Map with items
	 * @throws UnsupportedEncodingException if username can not be encoded or some string values can not be decoded
	 * @throws RecordException if internal error
	 */
	public static Result<Map<String,Object>> SYS__GETUAI(String usrnam, String[] itms) throws UnsupportedEncodingException, RecordException {
		return SYS$GETUAI(usrnam, itms);
	}
	/**
	 * Call SYS$PROCESS_SCAN to get first matching PID.
	 * @param code code for SYS$PROCESS_SCAN
	 * @param value value for SYS$PROCESS_SCAN
	 * @return return status code + PID
	 * @throws UnsupportedEncodingException if value can not be encoded
	 */
	public static Result<Integer> SYS__PROCESS_SCAN(int code, String value) throws UnsupportedEncodingException {
		return SYS$PROCESS_SCAN(code, value);
	}
	/**
	 * Call SYS$IDTOASC to get map of identifiers to identifier names.
	 * @param inclneg include negative (identifiers that are not UIC's)
	 * @return map of identifiers to identifier names
	 * @throws UnsupportedEncodingException if some identifier names can not be decoded
	 */
	public static Map<Integer,String> SYS__IDTOASC(boolean inclneg) throws UnsupportedEncodingException {
		return SYS$IDTOASC(inclneg);
	}
	/**
	 * Call SYS$GETDVIW to get a list of all items.
	 * @param devnam device name
	 * @return return status code + Java Map with items
	 * @throws UnsupportedEncodingException if device name can not be encoded or some string values can not be decoded
	 * @throws RecordException if internal error
	 */
	public static Result<Map<String,Object>> SYS__GETDVIW(String devnam) throws UnsupportedEncodingException, RecordException {
		return SYS$GETDVIW(devnam);
	}
	/**
	 * Call SYS$GETJPIW to get a list of all items.
	 * @param pid PID
	 * @return return status code + Java Map with items
	 * @throws UnsupportedEncodingException if some string values can not be encoded
	 * @throws RecordException if internal error
	 */
	public static Result<Map<String,Object>> SYS__GETJPIW(int pid) throws UnsupportedEncodingException, RecordException {
		return SYS$GETJPIW(pid);
	}
	/**
	 * Call SYS$GETUAI to get a list of all items.
	 * @param usrnam username
	 * @return return status code +  Java Map with items
	 * @throws UnsupportedEncodingException if username can not be encoded or some string values can not be decoded
	 * @throws RecordException if internal error
	 */
	public static Result<Map<String,Object>> SYS__GETUAI(String usrnam) throws UnsupportedEncodingException, RecordException {
		return SYS$GETUAI(usrnam);
	}
	/**
	 * Call SYS$IDTOASC to get list of all identifiers.
	 * @param inclneg include negative (identifiers that are not UIC's)
	 * @return list of all identifiers
	 * @throws UnsupportedEncodingException if some string values can not be encoded
	 */
	public static List<String> SYS__IDTOASC_AsList(boolean inclneg) throws UnsupportedEncodingException {
		return SYS$IDTOASC_AsList(inclneg);
	}
	/**
	 * Call SYS$PROCESS_SCAN to get all PID's and call SYS$GETJPIW for each PID to get list of items.
	 * @param code code for SYS$PROCESS_SCAN
	 * @param value value for SYS$PROCESS_SCAN
	 * @param itms array of items
	 * @return Java List of Java Map with items
	 * @throws UnsupportedEncodingException if value can not ne encoded or some string values can not be decoded
	 * @throws RecordException if internal error
	 */
	public static List<Map<String,Object>> SYS__PROCESS_SCAN_and_SYS__GETJPIW_AsList(String code, String value, String[] itms) throws UnsupportedEncodingException, RecordException {
		return SYS$PROCESS_SCAN_and_SYS$GETJPIW_AsList(code, value, itms);
	}
	/**
	 * Call SYS$PROCESS_SCAN to get all PID's and call SYS$GETJPIW for each PID to get list of all items.
	 * @param code code for SYS$PROCESS_SCAN
	 * @param value value for SYS$PROCESS_SCAN
	 * @return Java List of Java Map with items
	 * @throws UnsupportedEncodingException if value can not be encoded or some string values can not be decoded
	 * @throws RecordException if internal error
	 */
	public static List<Map<String,Object>> SYS__PROCESS_SCAN_and_SYS__GETJPIW_AsList(String code, String value) throws UnsupportedEncodingException, RecordException {
		return SYS$PROCESS_SCAN_and_SYS$GETJPIW_AsList(code, value);
	}
	/**
	 * Call SYS$QIOW.
	 * @param chan channel
	 * @param func function
	 * @param iosb IOSB
	 * @param p1 p1
	 * @param p2 p2
	 * @return return status code
	 * @throws RecordException if internal error
	 * @throws UnsupportedEncodingException if something can not be encoded or decoded
	 */
	public static Result<Void> SYS__QIOW(Word chan, int func, Block iosb, TypeWrapper<byte[]> p1, TypeWrapper<Integer> p2) throws RecordException, UnsupportedEncodingException {
		return SYS$QIOW(chan, func, iosb, p1, p2);
	}
	/**
	 * Call SYS$SETDDIR to get current default directory.
	 * @return return status code + current default directory
	 * @throws UnsupportedEncodingException if directory can not be decoded
	 */
	public static Result<String> SYS__SETDDIR() throws UnsupportedEncodingException {
		return SYS$SETDDIR();
	}
	/**
	 * Call SYS$SETDDIR to change current default directory.
	 * @param newdir new default directory
	 * @return return status code
	 * @throws UnsupportedEncodingException if directory can not be encoded
	 */
	public static Result<Void> SYS__SETDDIR(String newdir) throws UnsupportedEncodingException {
		return SYS$SETDDIR(newdir);
	}
	/**
	 * Class TERMINAL_IO represents the context (actually channel) for SYS$ASSIGN, SYS$QIOW and SYSDASSGN calls for terminal IO.
	 */
	public static class TERMINAL_IO extends dk.vajhoej.vms.call.lib.SYS.TERMINAL_IO {
		/**
		 * Call SYS$ASSIGN to open channel to default terminal (TT:).
		 * @return return status code
		 * @throws UnsupportedEncodingException if device name can not be decoded
		 */
		public Result<Void> SYS__ASSIGN() throws UnsupportedEncodingException {
			return SYS$ASSIGN();
		}
		/**
		 * Call SYS$ASSIGN to open channel to specified terminal.
		 * @param devnam terminal name
		 * @return return status code
		 * @throws UnsupportedEncodingException if device name can not be decoded
		 */
		public Result<Void> SYS__ASSIGN(String devnam) throws UnsupportedEncodingException {
			return SYS$ASSIGN(devnam);
		}
		/**
		 * Call SYS$QIOW to write string to terminal (puts functionality).
		 * @param s string to write
		 * @return return status code
		 * @throws RecordException if internal error
		 * @throws UnsupportedEncodingException if string can not be encoded
		 */
		public Result<Void> SYS__QIOW_puts(String s) throws RecordException, UnsupportedEncodingException {
			return SYS$QIOW_puts(s);
		}
		/**
		 * Call SYS$QIOW to read line from terminal (gets functionality).
		 * @return return status code + string read 
		 * @throws RecordException if internal error
		 * @throws UnsupportedEncodingException if string can not be decoded
		 */
		public Result<String> SYS__QIOW_gets() throws RecordException, UnsupportedEncodingException {
			return SYS$QIOW_gets();
		}
		/**
		 * Call SYS$QIOW to read single character from terminal (getch functionality).
		 * @return return status code + character read 
		 * @throws RecordException if internal error
		 * @throws UnsupportedEncodingException will never happen
		 */
		public Result<Character> SYS__QIOW_getch() throws RecordException, UnsupportedEncodingException {
			return SYS$QIOW_getch();
		}
		/**
		 * Call SYS$QIOW to check if there is input available to be read (kbhit functionality).
		 * @return  return status code + indication if input available
		 * @throws RecordException if internal error
		 * @throws UnsupportedEncodingException  will never happen
		 */
		public Result<Boolean> SYS__QIOW_kbhit() throws RecordException, UnsupportedEncodingException {
			return SYS$QIOW_kbhit();
		}
		/**
		 * Call LIB$DASSGN to close channel.
		 * @return return status code
		 */
		public Result<Void> SYS__DASSGN() {
			return SYS$DASSGN();
		}
	}
	/**
	 * Call SYS$TRNLNM to translate logical name.
	 * @param tblnam table name
	 * @param lognam logical name
	 * @return return status code + translate logical name
	 * @throws UnsupportedEncodingException if table or logical name can not be encoded or translation can not be decoded
	 */
	public static Result<String> SYS$TRNLNM(String tblnam, String lognam) throws UnsupportedEncodingException {
		return SYS$TRNLNM(tblnam, lognam);
	}
}
